import sys, getopt



def generate_problem_grace(index):
    filename = 'shift_'+str(index)+'.smv'
    out = open(filename, 'w')
    
    # Preamble
    print('-- Shift with n='+str(index)+' bits.', file=out)
    print('MODULE main', file=out)

    # EVAR
    print('EVAR', file=out)
    print('\t-- input bits', file=out)
    for i in range(0,index):
        print('\tin'+str(i)+' : boolean;', file=out)
    
    # VAR
    print('VAR', file=out)
    print('\t-- output bits', file=out)
    for i in range(0,index):
        print('\tout'+str(i)+' : boolean;', file=out)

    # GR1SPEC
    print('\nGR1SPEC', file=out)
    print('-- ENVIRONMENT', file=out)

    # ENVINIT
    print('\tENVINIT', file=out)
    print('\t\tTRUE', file=out)
    print('', file=out)

    # ENVTRANS
    print('\tENVTRANS', file=out)
    print('\t\t-- no shift', file=out)
    for i in range(0,index):
        end_str = ' ' if (i == index-1) else ' & '
        print('\t\t(out'+str(i)+' <-> X in'+str(i)+')'+end_str, file=out)
    
    # ENVFAIR
    print('\tENVFAIR', file=out)
    print('\t\tin'+str(index-1), file=out)


    # CONTROLLER
    print('-- CONTROLLER', file=out)

    # SYSINIT
    print('\tSYSINIT', file=out)
    print('\t\tTRUE', file=out)
    print('', file=out)

    # SYSTRANS
    print('\tSYSTRANS', file=out)
    print('\t\t-- shift', file=out)
    for i in range(0,index-1):
        print('\t\t(in'+str(i)+' <-> X out'+str(i+1)+') & ', file=out)
    print('\t\t(in'+str(index-1)+' <-> X out0)', file=out)


    # SYSFAIR
    print('\tSYSFAIR', file=out)
    print('\t\tout0', file=out)

    # result
    print('-- result: realizable', file=out)

    # close file
    out.close()






def generate_problem_slugs(index):
    filename = 'shift_'+str(index)+'.structuredslugs'
    out = open(filename, 'w')
    
    # Preamble
    print('# Shift with n='+str(index)+' bits.', file=out)

    # INPUT
    print('[INPUT]', file=out)
    print('\t# input bits', file=out)
    for i in range(0,index):
        print('\tin'+str(i), file=out)
    
    # OUTPUT
    print('[OUTPUT]', file=out)
    print('\t# output bits', file=out)
    for i in range(0,index):
        print('\tout'+str(i), file=out)

    # ENV_INIT
    print('[ENV_INIT]', file=out)
    print('\tTRUE', file=out)
    print('', file=out)

    # ENV_TRANS
    print('[ENV_TRANS]', file=out)
    print('\t# no shift', file=out)
    for i in range(0,index):
        print('\t\t(out'+str(i)+' <-> X in'+str(i)+')', file=out)
    
    # ENV_LIVENESS
    print('[ENV_LIVENESS]', file=out)
    print('\tin'+str(index-1), file=out)


    # CONTROLLER
    print('# CONTROLLER', file=out)

    # SYS_INIT
    print('[SYS_INIT]', file=out)
    print('\tTRUE', file=out)
    print('', file=out)

    # SYS_TRANS
    print('[SYS_TRANS]', file=out)
    print('\t# shift', file=out)
    for i in range(0,index-1):
        print('\t\t(in'+str(i)+' <-> X out'+str(i+1)+')', file=out)
    print('\t\t(in'+str(index-1)+' <-> X out0)', file=out)


    # SYS_LIVENESS
    print('[SYS_LIVENESS]', file=out)
    print('\t out0', file=out)

    # result
    print('# result: realizable', file=out)

    # close file
    out.close()
    



def print_help():
    print('generator-shift.py -i <index> -b <begin> -e <end> [-g] [-s]')
    print('-i <index>\t: generates the lift_controller problem with '
          'the specified index')
    print('-b\t\t: begin index')
    print('-e\t\t: end index')
    print('-g\t\t: dumps the problem in GRACE format')
    print('-s\t\t: dumps the problem in SLUGS format')




def main(argv):
    index = ''
    begin = ''
    end = ''
    grace = False
    slugs = False
    try:
        opts, args = getopt.getopt(argv,"hgsi:b:e:")
    except getopt.GetoptError:
        print_help()
        sys.exit()
    for opt, arg in opts:
        if opt == '-h':
            print_help()
            sys.exit()
        elif opt in ("-i", "--index"):
            index = arg
        elif opt in ("-b", "--begin"):
            begin = arg
        elif opt in ("-e", "--end"):
            end = arg
        elif opt in ("-g", "--grace"):
            grace = True
        elif opt in ("-s", "--slugs"):
            slugs = True
        else:
            print('Option %s not valid', opt)
            sys.exit(1)
    # check on the options
    index_number = 0
    if(not index and not begin and not end):
        print('*** Please specify the index with -i <index>')
        sys.exit()
    if index:
        index_number = int(index)
        if(index_number < 2):
            print('*** The index must be >= 2')
            sys.exit()
    if(index and (begin or end)):
        print('*** Please specify exactly one between -i and <-b,-e>')
        sys.exit()
    if(not index and (not begin or not end)):
        print('*** Please specify either -i or *both* -b and -e')
        sys.exit()
    if(begin and int(begin)<2):
        print('*** The starting index must be >= 2')
        sys.exit()
    if(not grace and not slugs):
        print('*** Please specify at least one between -g and -s')
        sys.exit()

    
    # main
    if index:
        if grace:
            generate_problem_grace(index_number)
        if slugs:
            generate_problem_slugs(index_number)
    else:
        for i in range(int(begin),int(end)+1):
            if grace:
                generate_problem_grace(i)
            if slugs:
                generate_problem_slugs(i)




if __name__ == "__main__":
   main(sys.argv[1:])
